import { IConfig } from "../interfaces/config.interface";
import { Position } from "../enums/position";
import { TooltipState } from "../enums/state";

export abstract class Tooltip {

	protected element: HTMLElement | null;
	protected content: HTMLElement | string;
	protected position: Position;
	protected tooltip: HTMLElement = document.createElement('div');
	protected state: TooltipState = TooltipState.CLOSED;
	protected disableClick: boolean;
	protected onClose: Function | null;
	// The amount of extra padding that the tooltip has for creating extra slack in the hover interaction
	protected outerPadding: number = 10;

	constructor(options: IConfig) {

		this.element = options.element;
		this.content = options.content;
		this.position = options.position || Position.TOP;
		this.disableClick = options.disableClick || false;
		this.onClose = options.onClose || null;
		this.init();
	}

	private init(): void {

		if(this.element === null) {

			console.error("Tooltip: No element was provided!");
			return;
		}

		this.element.style.position = "relative";

		this.createTooltipElement();

		this.element.appendChild(this.tooltip);
		
		this.addEventListeners();
	}

	abstract createTooltipElement(): void;

	abstract addEventListeners(): void;

	public open(): void {
		this.tooltip.style.display = 'block';
		this.state = TooltipState.OPEN;
		window.requestAnimationFrame(this.checkBounds.bind(this));
	}

	public close(): void {
		this.tooltip.style.display = 'none';
		this.state = TooltipState.CLOSED;
	}

	private checkBounds(): void {

		this.tooltip.classList.remove('Left', 'Right');

		let bounds: any = this.tooltip.getBoundingClientRect();

		if(bounds.y < 0 || this.position === Position.BOTTOM) {
			this.tooltip.classList.add('Bottom');
			//Refresh bounds object since some properties will have changed
			bounds = this.tooltip.getBoundingClientRect();
		}

		if((bounds.y + bounds.height + this.outerPadding) > window.innerHeight) {
			this.tooltip.classList.remove('Bottom');
		}

		if(window.innerWidth < bounds.width) {
			return;
		}

		const tooltipArrow: HTMLElement = this.tooltip.querySelector('.TooltipArrow') as HTMLElement;

		if(!tooltipArrow) { return; }

		if(bounds.x < 0) {
			// Re-position the tooltip so it doesn't fall outside of the viewport
			this.tooltip.style.left = `calc(50% + ${Math.abs(bounds.x)}px)`;
			// Re-position the arrow the same amount in the opposite direction so it stays centered
			tooltipArrow.style.left = `calc(50% - ${Math.abs(bounds.x)}px)`;
		}

		if((bounds.x + bounds.width) > document.documentElement.clientWidth) {
			// Re-position the tooltip so it doesn't fall outside of the viewport
			this.tooltip.style.left = `calc(50% - ${Math.abs((bounds.x + bounds.width) - document.documentElement.clientWidth)}px)`;
			// Re-position the arrow the same amount in the opposite direction so it stays centered
			tooltipArrow.style.left = `calc(50% + ${Math.abs((bounds.x + bounds.width) - document.documentElement.clientWidth)}px)`;
		}
	}
}