import { ContextualTooltip } from "../../../../../src/domain/contextual-tooltip";
import { TooltipState } from "../../../../../src/enums/state";
import { requestAnimationFrameMock } from "../../../mocks/mock_requestAnimationFrame";
import { Position } from "../../../../../src/enums/position";

let testInstance;
let testElement = document.createElement('div');
let testArrow = document.createElement('div');

describe('tooltip.ts', () => {

	beforeEach(() => {

		requestAnimationFrameMock.reset();

		testInstance = new ContextualTooltip({
			element: testElement,
			content: "Test"
		});
	});

	it('init(): Returns when given element equals null.', () => {

		//given
		testInstance.element = null;
		const spy = jest.spyOn(testInstance, "init");

		//when
		testInstance.init();

		//then
		expect(spy).toHaveReturned();
	});

	it('init(): Succesfully initializes the tooltip instance.', () => {

		//given
		testInstance.element = {
			style: {
				position: ""
			},
			addEventListener: jest.fn(),
			appendChild: jest.fn(),
			querySelector() {
				return testArrow;
			}
		};
		const spyCreateTooltipElement = jest.spyOn(testInstance, "createTooltipElement");
		const spyAddEventListeners = jest.spyOn(testInstance, "addEventListeners");

		//when
		testInstance.init();

		//then
		expect(spyCreateTooltipElement).toHaveBeenCalled();
		expect(spyAddEventListeners).toHaveBeenCalled();
		expect(testInstance.element.appendChild).toHaveBeenCalled();
		expect(testInstance.element.style.position).toEqual('relative');
	});

	it('open(): Succesfully opens the tooltip instance.', () => {

		//given
		testInstance.tooltip = {
			style: {
				display: ""
			}
		};
		testInstance.checkBounds = jest.fn();

		//when
		testInstance.open();
		requestAnimationFrameMock.triggerNextAnimationFrame();

		//then
		expect(testInstance.checkBounds).toHaveBeenCalled();
		expect(testInstance.state).toEqual(TooltipState.OPEN);
		expect(testInstance.tooltip.style.display).toEqual('block');
	});

	it('close(): Succesfully closes the tooltip instance.', () => {

		//given
		testInstance.tooltip = {
			style: {
				display: ""
			}
		};

		//when
		testInstance.close();

		//then
		expect(testInstance.state).toEqual(TooltipState.CLOSED);
		expect(testInstance.tooltip.style.display).toEqual('none');
	});

	it('checkBounds(): Doesn\'t shift positioning when it\'s not needed.', () => {

		//given
		testInstance.tooltip = {
			getBoundingClientRect() {
				return {
					x: 100,
					y: 100,
					width: 100,
					height: 100
				}
			},
			style: {
				left: ''
			},
			classList: {
				remove: jest.fn(),
				add: jest.fn()
			},
			querySelector() {
				return testArrow;
			}
		};

		//when
		testInstance.checkBounds();

		//then
		expect(testInstance.tooltip.classList.remove).toHaveBeenCalledWith('Left', 'Right');
		expect(testInstance.tooltip.classList.add).toHaveBeenCalledTimes(0);
	});

	it('checkBounds(): Shifts the tooltip down when it\'s configured that way or it doesn\'t fit on top.', () => {

		//given
		testInstance.tooltip = {
			getBoundingClientRect() {
				return {
					x: 100,
					y: -100,
					width: 100,
					height: 100
				}
			},
			style: {
				left: ''
			},
			classList: {
				remove: jest.fn(),
				add: jest.fn()
			},
			querySelector() {
				return testArrow;
			}
		};

		//when
		testInstance.checkBounds();

		//then
		expect(testInstance.tooltip.classList.add).toHaveBeenCalledWith('Bottom');

		//given
		testInstance.tooltip = {
			getBoundingClientRect() {
				return {
					x: 100,
					y: 100,
					width: 100,
					height: 100
				}
			},
			style: {
				left: ''
			},
			classList: {
				remove: jest.fn(),
				add: jest.fn()
			},
			querySelector() {
				return testArrow;
			}
		};
		testInstance.position = Position.BOTTOM;

		//when
		testInstance.checkBounds();

		//then
		expect(testInstance.tooltip.classList.add).toHaveBeenCalledWith('Bottom');
	});

	it('checkBounds(): Shifts the tooltip up when it doesn\'t fit on the bottom.', () => {

		//given
		global.innerHeight = 200;
		testInstance.tooltip = {
			getBoundingClientRect() {
				return {
					x: 100,
					y: 100,
					width: 100,
					height: 100
				}
			},
			style: {
				left: ''
			},
			classList: {
				remove: jest.fn(),
				add: jest.fn()
			},
			querySelector() {
				return testArrow;
			}
		};

		//when
		testInstance.checkBounds();

		//then
		expect(testInstance.tooltip.classList.remove).toHaveBeenCalledWith('Bottom');
	});

	it('checkBounds(): Returns when the tooltip is wider than the viewport', () => {

		//given
		global.innerWidth = 90;
		testInstance.tooltip = {
			getBoundingClientRect() {
				return {
					x: 100,
					y: 100,
					width: 100,
					height: 100
				}
			},
			style: {
				left: ''
			},
			classList: {
				remove: jest.fn(),
				add: jest.fn()
			},
			querySelector() {
				return testArrow;
			}
		};
		const spy = jest.spyOn(testInstance, "checkBounds");

		//when
		testInstance.checkBounds();

		//then
		expect(spy).toHaveReturned();
	});

	it('checkBounds(): Shifts the tooltip left when there is no space on the right.', () => {

		//given
		global.innerWidth = 1000;
		testInstance.tooltip = {
			getBoundingClientRect() {
				return {
					x: -100,
					y: 100,
					width: 100,
					height: 100
				}
			},
			style: {
				left: ''
			},
			classList: {
				remove: jest.fn(),
				add: jest.fn()
			},
			querySelector() {
				return testArrow;
			}
		};

		//when
		testInstance.checkBounds();

		//then
		expect(testInstance.tooltip.style.left).toEqual('calc(50% + 100px)');
		expect(testArrow.style.left).toEqual('calc(50% - 100px)');
	});

	it('checkBounds(): Shifts the tooltip right when there is no space on the left.', () => {

		//given
		global.innerWidth = 180;
		testInstance.tooltip = {
			getBoundingClientRect() {
				return {
					x: 100,
					y: 100,
					width: 100,
					height: 100
				}
			},
			style: {
				left: ''
			},
			classList: {
				remove: jest.fn(),
				add: jest.fn()
			},
			querySelector() {
				return testArrow;
			}
		};

		//when
		testInstance.checkBounds();

		//then
		expect(testInstance.tooltip.style.left).toEqual('calc(50% - 200px)');
		expect(testArrow.style.left).toEqual('calc(50% + 200px)');
	});
});